@@ -16,6 +16,7 @@
 
 #include "DMRaid.h"
 #include "BlockSpecial.h"
+#include "OperationDetail.h"
 #include "Partition.h"
 #include "Utils.h"
 
@@ -30,6 +31,7 @@
 namespace GParted
 {
 
+
 // Data model:
 // dmraid_cache_initialized - Has the cache been loaded?
 // dmraid_found             - Is the "dmraid" command available?
@@ -54,35 +56,13 @@ std::vector<Glib::ustring> DMRaid::dmraid_devices ;
 std::vector<DMRaid_Member> DMRaid::dmraid_member_cache;
 
 
-DMRaid::DMRaid()
+void DMRaid::load_cache()
 {
-	//Ensure that cache has been loaded at least once
-	if ( ! dmraid_cache_initialized )
-	{
-		dmraid_cache_initialized = true ;
-		set_commands_found() ;
-		load_dmraid_cache() ;
-	}
+	set_commands_found();
+	load_dmraid_cache();
+	dmraid_cache_initialized = true;
 }
 
-DMRaid::DMRaid( const bool & do_refresh )
-{
-	//Ensure that cache has been loaded at least once
-	if ( ! dmraid_cache_initialized )
-	{
-		dmraid_cache_initialized = true ;
-		set_commands_found() ;
-		if ( do_refresh == false )
-			load_dmraid_cache() ;
-	}
-
-	if ( do_refresh )
-		load_dmraid_cache() ;
-}
-
-DMRaid::~DMRaid()
-{
-}
 
 void DMRaid::load_dmraid_cache()
 {
@@ -160,11 +140,11 @@ bool DMRaid::is_dmraid_device( const Glib::ustring & dev_path )
 		}
 
 		//Also check for a symbolic link if device not yet found
-		if ( ! device_found && file_test( dev_path, Glib::FILE_TEST_IS_SYMLINK ) )
+		if (! device_found && Glib::file_test(dev_path, Glib::FILE_TEST_IS_SYMLINK))
 		{
 			//Path is a symbolic link so find real path
-			char * rpath = realpath( dev_path.c_str(), NULL );
-			if ( rpath != NULL )
+			char* rpath = realpath(dev_path.c_str(), nullptr);
+			if (rpath != nullptr)
 			{
 				Glib::ustring tmp_path = rpath;
 				free( rpath );
@@ -179,43 +159,27 @@ bool DMRaid::is_dmraid_device( const Glib::ustring & dev_path )
 	return device_found ;
 }
 
-//FIXME:  Copy of code from FileSystem.cc.
-//        This should be replaced when a general execute_command with asynchronous updates is built.
-int DMRaid::execute_command( const Glib::ustring & command, OperationDetail & operationdetail ) 
-{
-	Glib::ustring output, error ;
 
-	operationdetail .add_child( OperationDetail( command, STATUS_NONE, FONT_BOLD_ITALIC ) ) ;
-
-	int exit_status = Utils::execute_command( command, output, error );
-
-	if ( ! output .empty() )
-		operationdetail .get_last_child() .add_child( OperationDetail( output, STATUS_NONE, FONT_ITALIC ) ) ;
-	
-	if ( ! error .empty() )
-		operationdetail .get_last_child() .add_child( OperationDetail( error, STATUS_NONE, FONT_ITALIC ) ) ;
-
-	return exit_status ;
-}
-
-void DMRaid::get_devices( std::vector<Glib::ustring> & device_list )
+std::vector<Glib::ustring> DMRaid::get_devices()
 {
 	//Retrieve list of dmraid devices
-	device_list .clear() ;
+	std::vector<Glib::ustring> device_list;
 
 	for ( unsigned int k=0; k < dmraid_devices .size(); k++ )
 		device_list.push_back( DEV_MAPPER_PATH + dmraid_devices[k] );
+
+	return device_list;
 }
 
+
 Glib::ustring DMRaid::get_dmraid_name( const Glib::ustring & dev_path )
 {
 	//Retrieve name of dmraid device
-	Glib::ustring dmraid_name = "" ;
-	Glib::ustring regexp = "" ;
+	Glib::ustring dmraid_name;
 
 	for ( unsigned int k=0; k < dmraid_devices .size(); k++ )
 	{
-		regexp = ".*(" + dmraid_devices[k] + ").*" ;
+		Glib::ustring regexp = ".*(" + dmraid_devices[k] + ").*";
 		if ( Utils::regexp_label( dev_path, regexp ) == dmraid_devices[k] )
 			dmraid_name = dmraid_devices[k] ;
 	}
@@ -230,11 +194,11 @@ Glib::ustring DMRaid::get_dmraid_name( const Glib::ustring & dev_path )
 				dmraid_name = dmraid_devices[k] ;
 
 		//Also check for a symbolic link if dmraid_name not yet found
-		if ( dmraid_name .empty() && file_test( dev_path, Glib::FILE_TEST_IS_SYMLINK ) )
+		if (dmraid_name.empty() && Glib::file_test(dev_path, Glib::FILE_TEST_IS_SYMLINK))
 		{
 			//Path is a symbolic link so find real path
-			char * rpath = realpath( dev_path.c_str(), NULL );
-			if ( rpath != NULL )
+			char* rpath = realpath(dev_path.c_str(), nullptr);
+			if (rpath != nullptr)
 			{
 				Glib::ustring tmp_path = rpath;
 				free( rpath );
@@ -249,14 +213,16 @@ Glib::ustring DMRaid::get_dmraid_name( const Glib::ustring & dev_path )
 	return dmraid_name ;
 }
 
-void DMRaid::get_dmraid_dir_entries( const Glib::ustring & dev_path, std::vector<Glib::ustring> & dir_list )
+
+std::vector<Glib::ustring> DMRaid::get_dmraid_dir_entries(const Glib::ustring& dev_path)
 {
 	//Build list of all device entries matching device path
 
 	Glib::ustring dmraid_name = get_dmraid_name( dev_path ) ;
 
 	//Loop through the entries in the directory
-	Glib::ustring filename = "" ;
+	std::vector<Glib::ustring> dir_list;
+	Glib::ustring filename;
 	Glib::Dir dir( DEV_MAPPER_PATH );
 	while ( ( filename = dir .read_name() ) != "" )
 	{
@@ -266,34 +232,35 @@ void DMRaid::get_dmraid_dir_entries( const Glib::ustring & dev_path, std::vector
 		if ( Utils::regexp_label( filename, "^(" + dmraid_name + ")" ) == dmraid_name )
 			dir_list .push_back( filename ) ;
 	}
+
+	return dir_list;
 }
 
+
 int DMRaid::get_partition_number( const Glib::ustring & partition_name )
 {
 	Glib::ustring dmraid_name = get_dmraid_name( partition_name ) ;
 	return std::atoi( Utils::regexp_label( partition_name, dmraid_name + "p?([0-9]+)" ) .c_str() ) ;
 }
 
+
 Glib::ustring DMRaid::get_udev_dm_name( const Glib::ustring & dev_path )
 {
+	Glib::ustring dm_name;
+	if (! udevadm_found)
+		return dm_name;
+
 	//Retrieve DM_NAME of device using udev information
-	Glib::ustring output = "" ;
-	Glib::ustring error  = "" ;
-	Glib::ustring dm_name = "" ;
-
-	if (udevadm_found)
-		Utils::execute_command( "udevadm info --query=all --name=" + Glib::shell_quote( dev_path ),
-		                        output, error, true );
-
-	if ( ! output .empty() )
-	{
-		Glib::ustring regexp = "^E: DM_NAME=([^\n]*)$" ;
-		dm_name = Utils::regexp_label( output, regexp ) ;
-	}
+	Glib::ustring output;
+	Glib::ustring error;
+	Utils::execute_command("udevadm info --query=property --name=" + Glib::shell_quote(dev_path),
+	                       output, error, true);
+	dm_name = Utils::regexp_label(output, "^DM_NAME=([^\n]*)$");
 
 	return dm_name ;
 }
 
+
 Glib::ustring DMRaid::make_path_dmraid_compatible( Glib::ustring partition_path )
 {
 	//The purpose of this method is to ensure that the partition name matches
@@ -351,47 +318,46 @@ Glib::ustring DMRaid::make_path_dmraid_compatible( Glib::ustring partition_path
 	return partition_path ;
 }
 
+
 bool DMRaid::create_dev_map_entries( const Partition & partition, OperationDetail & operationdetail )
 {
 	//Create all missing dev mapper entries for a specified device.
 
-	bool exit_status = true ;
+	bool success = true;
 
 	/*TO TRANSLATORS: looks like  create missing /dev/mapper entries */ 
-	Glib::ustring tmp = Glib::ustring::compose( _("create missing %1 entries"), DEV_MAPPER_PATH );
-	operationdetail .add_child( OperationDetail( tmp ) );
+	operationdetail.add_child(OperationDetail(
+	                        Glib::ustring::compose(_("create missing %1 entries"), DEV_MAPPER_PATH)));
+	OperationDetail& child_od = operationdetail.get_last_child();
 
 	//Newer dmraid defaults to always inserting the letter 'p' between the device name
 	//  and the partition number.
-	if ( execute_command( "dmraid -ay -P \"\" -v", operationdetail.get_last_child() ) )
-		exit_status = false;  // command failed
+	if (child_od.execute_command("dmraid -ay -P \"\" -v"))
+		success = false;  // command failed
 
-	operationdetail.get_last_child().set_success_and_capture_errors( exit_status );
-	return exit_status ;
+	child_od.set_success_and_capture_errors(success);
+	return success;
 }
 
+
 bool DMRaid::create_dev_map_entries( const Glib::ustring & dev_path )
 {
 	//Create all missing dev mapper entries for a specified device.
 
 	Glib::ustring output, error ;
-	bool exit_status = true ;
 
 	//Newer dmraid defaults to always inserting the letter 'p' between the device name
 	//  and the partition number.
-	if ( Utils::execute_command( "dmraid -ay -P \"\" -v", output, error, true ) )
-		exit_status = false;  // command failed
-
-	return exit_status ;
+	return ! Utils::execute_command("dmraid -ay -P \"\" -v", output, error, true);
 }
 
-void DMRaid::get_affected_dev_map_entries( const Partition & partition, std::vector<Glib::ustring> & affected_entries )
+
+std::vector<Glib::ustring> DMRaid::get_affected_dev_map_entries(const Partition& partition)
 {
 	//Build list of affected /dev/mapper entries when a partition is to be deleted.
 
 	//Retrieve list of matching directory entries
-	std::vector<Glib::ustring> dir_list ;
-	get_dmraid_dir_entries( partition .device_path, dir_list );
+	std::vector<Glib::ustring> dir_list = get_dmraid_dir_entries(partition.device_path);
 
 	//All partition numbers equal to the number of the partition to be deleted
 	//  will be affected.
@@ -399,6 +365,7 @@ void DMRaid::get_affected_dev_map_entries( const Partition & partition, std::vec
 	//  partition numbers greater than the number of the partition to be deleted
 	//  will be affected.
 	Glib::ustring dmraid_name = get_dmraid_name( partition .device_path ) ;
+	std::vector<Glib::ustring> affected_entries;
 	for ( unsigned int k=0; k < dir_list .size(); k++ )
 	{
 		if ( Utils::regexp_label( dir_list[k], "^(" + dmraid_name + ")" ) == dmraid_name )
@@ -410,18 +377,21 @@ void DMRaid::get_affected_dev_map_entries( const Partition & partition, std::vec
 				affected_entries .push_back( dir_list[k] ) ;
 		}
 	}
+
+	return affected_entries;
 }
 
-void DMRaid::get_partition_dev_map_entries( const Partition & partition, std::vector<Glib::ustring> & partition_entries )
+
+std::vector<Glib::ustring> DMRaid::get_partition_dev_map_entries(const Partition& partition)
 {
 	//Build list of all /dev/mapper entries for a partition.
 
 	//Retrieve list of matching directory entries
-	std::vector<Glib::ustring> dir_list ;
-	get_dmraid_dir_entries( partition .device_path, dir_list );
+	std::vector<Glib::ustring> dir_list = get_dmraid_dir_entries(partition.device_path);
 
 	//Retrieve all partition numbers equal to the number of the partition.
 	Glib::ustring dmraid_name = get_dmraid_name( partition .device_path ) ;
+	std::vector<Glib::ustring> partition_entries;
 	for ( unsigned int k=0; k < dir_list .size(); k++ )
 	{
 		if ( Utils::regexp_label( dir_list[k], "^(" + dmraid_name + ")" ) == dmraid_name )
@@ -431,77 +401,80 @@ void DMRaid::get_partition_dev_map_entries( const Partition & partition, std::ve
 				partition_entries .push_back( dir_list[k] ) ;
 		}
 	}
+
+	return partition_entries;
 }
 
+
 bool DMRaid::delete_affected_dev_map_entries( const Partition & partition, OperationDetail & operationdetail )
 {
 	// Delete all affected dev mapper entries (logical partitions >= specified partition)
 
-	std::vector<Glib::ustring> affected_entries ;
 	Glib::ustring command ;
-	bool exit_status = true ;
+	bool success = true;
 
 	/*TO TRANSLATORS: looks like  delete affected /dev/mapper entries */ 
-	Glib::ustring tmp = Glib::ustring::compose( _("delete affected %1 entries"), DEV_MAPPER_PATH );
-	operationdetail .add_child( OperationDetail( tmp ) );
+	operationdetail.add_child(OperationDetail(
+	                        Glib::ustring::compose(_("delete affected %1 entries"), DEV_MAPPER_PATH)));
+	OperationDetail& child_od = operationdetail.get_last_child();
 
-	get_affected_dev_map_entries( partition, affected_entries ) ;
+	std::vector<Glib::ustring> affected_entries = get_affected_dev_map_entries(partition);
 
 	for ( unsigned int k=0; k < affected_entries .size(); k++ )
 	{
 		command = "dmsetup remove " + Glib::shell_quote( DEV_MAPPER_PATH + affected_entries[k] );
-		if ( execute_command( command, operationdetail .get_last_child() ) )
-			exit_status = false ;	//command failed
+		if (child_od.execute_command(command))
+			success = false;  // command failed
 	}
 
-	operationdetail.get_last_child().set_success_and_capture_errors( exit_status );
-	return exit_status ;
+	child_od.set_success_and_capture_errors(success);
+	return success;
 }
 
+
 bool DMRaid::delete_dev_map_entry( const Partition & partition, OperationDetail & operationdetail )
 {
 	//Delete a single partition which may be represented by multiple dev mapper entries
-	bool exit_status = true ;
+	bool success = true;
 
 	/*TO TRANSLATORS: looks like  delete /dev/mapper entry */ 
-	Glib::ustring tmp = Glib::ustring::compose( _("delete %1 entry"), DEV_MAPPER_PATH );
-	operationdetail .add_child( OperationDetail( tmp ) );
+	operationdetail.add_child(OperationDetail(
+	                        Glib::ustring::compose(_("delete %1 entry"), DEV_MAPPER_PATH)));
+	OperationDetail& child_od = operationdetail.get_last_child();
 
-	std::vector<Glib::ustring> partition_entries ;
-	get_partition_dev_map_entries( partition, partition_entries ) ;
-	
+	std::vector<Glib::ustring> partition_entries = get_partition_dev_map_entries(partition);
 	for ( unsigned int k = 0; k < partition_entries .size(); k++ )
 	{
 		Glib::ustring command = "dmsetup remove " + Glib::shell_quote( partition_entries[k] );
-		if ( execute_command( command, operationdetail .get_last_child() ) )
-			exit_status = false ;	//command failed
+		if (child_od.execute_command(command))
+			success = false;  // command failed
 	}
 
-	operationdetail.get_last_child().set_success_and_capture_errors( exit_status );
-	return exit_status ;
+	child_od.set_success_and_capture_errors(success);
+	return success;
 }
 
+
 bool DMRaid::purge_dev_map_entries( const Glib::ustring & dev_path )
 {
 	//Delete all dev mapper entries for dmraid device
 
-	std::vector<Glib::ustring> dir_list ;
+	std::vector<Glib::ustring> dir_list = get_dmraid_dir_entries(dev_path);
 	Glib::ustring command ;
 	Glib::ustring output, error ;
-	bool exit_status = true ;
-
-	get_dmraid_dir_entries( dev_path, dir_list ) ;
+	bool success = true;
 
 	for ( unsigned int k=0; k < dir_list .size(); k++ )
 	{
 		command = "dmsetup remove " + Glib::shell_quote( DEV_MAPPER_PATH + dir_list[k] );
 		if ( Utils::execute_command( command, output, error, true ) )
-			exit_status = false ;	//command failed
+			success = false;  // command failed
 	}
 
-	return exit_status ;
+	return success;
 }
 
+
 bool DMRaid::update_dev_map_entry( const Partition & partition, OperationDetail & operationdetail )
 {
 	//Update dmraid entry by first deleting the entry then recreating the entry
@@ -510,20 +483,21 @@ bool DMRaid::update_dev_map_entry( const Partition & partition, OperationDetail
 	if (partition.type == TYPE_EXTENDED)
 		return true ;
 
-	bool exit_status = true ;
+	bool success = true;
 
 	/*TO TRANSLATORS: looks like  update /dev/mapper entry */ 
-	Glib::ustring tmp = Glib::ustring::compose( _("update %1 entry"), DEV_MAPPER_PATH );
-	operationdetail .add_child( OperationDetail( tmp ) );
+	operationdetail.add_child(OperationDetail(
+	                        Glib::ustring::compose(_("update %1 entry"), DEV_MAPPER_PATH)));
+	OperationDetail& child_od = operationdetail.get_last_child();
 
-	if( ! delete_dev_map_entry( partition , operationdetail .get_last_child() ) )
-		exit_status = false ;	//command failed
+	if (! delete_dev_map_entry(partition, child_od))
+		success = false;  // command failed
 
-	if( ! create_dev_map_entries( partition , operationdetail .get_last_child() ) )
-		exit_status = false ;	//command failed
+	if (! create_dev_map_entries(partition, child_od))
+		success = false;  // command failed
 
-	operationdetail.get_last_child().set_success_and_capture_errors( exit_status );
-	return exit_status ;
+	child_od.set_success_and_capture_errors(success);
+	return success;
 }
 
 
@@ -572,11 +546,7 @@ std::vector<Glib::ustring> DMRaid::lookup_dmraid_members(const Glib::ustring& ar
 	Glib::ustring error;
 	Utils::execute_command("udevadm info --query=name " + Glib::shell_quote(array),
 	                       output, error, true);
-
-	// Strip terminating new line from output.
-	size_t len = output.length();
-	if (len > 0 && output[len-1] == '\n')
-		output.resize(len-1);
+	output = Utils::trim_trailing_new_line(output);
 
 	if (output.empty())
 		return members;  // Empty vector
@@ -608,4 +578,4 @@ const DMRaid_Member& DMRaid::get_cache_entry_by_member(const Glib::ustring& memb
 }
 
 
-}//GParted
+}  // namespace GParted
