@@ -36,6 +36,7 @@
 #include <gtkmm/enums.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/stockitem.h>
+#include <sigc++/signal.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -47,6 +48,77 @@
 namespace GParted
 {
 
+
+namespace  // unnamed
+{
+
+
+class CommandStatus
+{
+public:
+	bool        running;
+	int         pipecount;
+	int         exit_status;
+	bool        foreground;
+	Glib::Mutex mutex;
+	Glib::Cond  cond;
+
+	void store_exit_status(GPid pid, int status);
+	void execute_command_eof();
+};
+
+
+void CommandStatus::store_exit_status(GPid pid, int status)
+{
+	exit_status = Utils::decode_wait_status(status);
+	running = false;
+	if (pipecount == 0)  // pipes finished first
+	{
+		if (foreground)
+			Gtk::Main::quit();
+		else {
+			mutex.lock();
+			cond.signal();
+			mutex.unlock();
+		}
+	}
+	Glib::spawn_close_pid(pid);
+}
+
+
+void CommandStatus::execute_command_eof()
+{
+	if (--pipecount)
+		return;  // wait for second pipe to eof
+	if (! running)  // already got exit status
+	{
+		if (foreground)
+			Gtk::Main::quit();
+		else {
+			mutex.lock();
+			cond.signal();
+			mutex.unlock();
+		}
+	}
+}
+
+
+static void set_C_locale()
+{
+	setenv("LC_ALL", "C", 1);
+}
+
+
+static void _store_exit_status(GPid pid, gint status, gpointer data)
+{
+	CommandStatus *sp = (CommandStatus *)data;
+	sp->store_exit_status(pid, status);
+}
+
+
+}  // unnamed namespace
+
+
 const Glib::ustring DEV_MAPPER_PATH = "/dev/mapper/";
 
 Sector Utils::round( double double_value )
@@ -67,7 +139,7 @@ Gtk::Label * Utils::mk_label( const Glib::ustring & text
                             , Gtk::Align valign
                             )
 {
-	Gtk::Label *label = manage(new Gtk::Label(text));
+	Gtk::Label* label = Gtk::manage(new Gtk::Label(text));
 
 	label->set_xalign(0.0);
 	label->set_valign(valign);
@@ -85,7 +157,7 @@ Gtk::Label * Utils::mk_label( const Glib::ustring & text
 
 Gtk::Image* Utils::mk_image(const Gtk::StockID& stock_id, Gtk::IconSize icon_size)
 {
-	Gtk::Image *image = Gtk::manage(new Gtk::Image(stock_id, icon_size));
+	Gtk::Image* image = Gtk::manage(new Gtk::Image(stock_id, icon_size));
 
 	// Ensure icon size
 	int width = 0;
@@ -105,6 +177,8 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::mk_pixbuf(Gtk::Widget& widget,
                                            Gtk::IconSize icon_size)
 {
 	Glib::RefPtr<Gdk::Pixbuf> theme_icon = widget.render_icon_pixbuf(stock_id, icon_size);
+	if (! theme_icon)
+		return theme_icon;
 
 	// Ensure icon size
 	int width = 0;
@@ -443,7 +517,7 @@ const Glib::ustring Utils::get_filesystem_software(FSType fstype)
 		case FS_EXFAT:      return "exfatprogs";
 		case FS_EXT2:       return "e2fsprogs";
 		case FS_EXT3:       return "e2fsprogs";
-		case FS_EXT4:       return "e2fsprogs v1.41+";
+		case FS_EXT4:       return "e2fsprogs";
 		case FS_F2FS:       return "f2fs-tools";
 		case FS_FAT16:      return "dosfstools, mtools";
 		case FS_FAT32:      return "dosfstools, mtools";
@@ -625,64 +699,6 @@ int Utils::execute_command( const Glib::ustring & command )
 	Glib::ustring dummy ;
 	return execute_command(command, nullptr, dummy, dummy);
 }
-
-class CommandStatus
-{
-public:
-	bool running;
-	int pipecount;
-	int exit_status;
-	bool foreground;
-	Glib::Mutex mutex;
-	Glib::Cond cond;
-	void store_exit_status( GPid pid, int status );
-	void execute_command_eof();
-};
-
-void CommandStatus::store_exit_status( GPid pid, int status )
-{
-	exit_status = Utils::decode_wait_status( status );
-	running = false;
-	if (pipecount == 0) // pipes finished first
-	{
-		if (foreground)
-			Gtk::Main::quit();
-		else {
-			mutex.lock();
-			cond.signal();
-			mutex.unlock();
-		}
-	}
-	Glib::spawn_close_pid( pid );
-}
-
-void CommandStatus::execute_command_eof()
-{
-	if (--pipecount)
-		return; // wait for second pipe to eof
-	if ( !running ) // already got exit status
-	{
-		if (foreground)
-			Gtk::Main::quit();
-		else {
-			mutex.lock();
-			cond.signal();
-			mutex.unlock();
-		}
-	}
-}
-
-static void set_locale()
-{
-	setenv( "LC_ALL", "C", 1 );
-}
-
-static void _store_exit_status( GPid pid, gint status, gpointer data )
-{
-	CommandStatus *sp = (CommandStatus *)data;
-	sp->store_exit_status( pid, status );
-}
-
 int Utils::execute_command( const Glib::ustring & command,
 			    Glib::ustring & output,
 			    Glib::ustring & error,
@@ -711,7 +727,7 @@ int Utils::execute_command( const Glib::ustring & command,
 			std::string( "." ),
 			Glib::shell_parse_argv( command ),
 			Glib::SPAWN_DO_NOT_REAP_CHILD | Glib::SPAWN_SEARCH_PATH,
-			use_C_locale ? sigc::ptr_fun( set_locale ) : sigc::slot< void >(),
+			use_C_locale ? sigc::ptr_fun(set_C_locale) : sigc::slot<void>(),
 			&pid,
 			(input != nullptr) ? &in : 0,
 			&out,
@@ -966,7 +982,7 @@ bool Utils::is_dev_busy(const Glib::ustring& path)
 const Glib::ustring& Utils::first_directory(const std::vector<Glib::ustring>& paths)
 {
 	for (unsigned int i = 0; i < paths.size(); i++)
-		if (file_test(paths[i], Glib::FILE_TEST_IS_DIR))
+		if (Glib::file_test(paths[i], Glib::FILE_TEST_IS_DIR))
 			return paths[i];
 
 	static const Glib::ustring not_found;
@@ -1020,4 +1036,5 @@ bool Utils::get_kernel_version( int & major_ver, int & minor_ver, int & patch_ve
 	return success ;
 }
 
-} //GParted..
+
+}  // namespace GParted
