@@ -33,6 +33,8 @@
  */
 
 
+#include "common.h"
+#include "insertion_operators.h"
 #include "GParted_Core.h"
 #include "FileSystem.h"
 #include "OperationDetail.h"
@@ -62,60 +64,6 @@ namespace GParted
 {
 
 
-// Hacky XML parser which strips italic and bold markup added in
-// OperationDetail::set_description() and reverts just these 5 characters &<>'" encoded by
-// Glib::Markup::escape_text() -> g_markup_escape_text() -> append_escaped_text().
-Glib::ustring strip_markup(const Glib::ustring& str)
-{
-	size_t len = str.length();
-	size_t i = 0;
-	Glib::ustring ret;
-	ret.reserve(len);
-	while (i < len)
-	{
-		if (str.compare(i, 3, "<i>") == 0)
-			i += 3;
-		else if (str.compare(i, 4, "</i>") == 0)
-			i += 4;
-		else if (str.compare(i, 3, "<b>") == 0)
-			i += 3;
-		else if (str.compare(i, 4, "</b>") == 0)
-			i += 4;
-		else if (str.compare(i, 5, "&amp;") == 0)
-		{
-			ret.push_back('&');
-			i += 5;
-		}
-		else if (str.compare(i, 4, "&lt;") == 0)
-		{
-			ret.push_back('<');
-			i += 4;
-		}
-		else if (str.compare(i, 4, "&gt;") == 0)
-		{
-			ret.push_back('>');
-			i += 4;
-		}
-		else if (str.compare(i, 6, "&apos;") == 0)
-		{
-			ret.push_back('\'');
-			i += 6;
-		}
-		else if (str.compare(i, 6, "&quot;") == 0)
-		{
-			ret.push_back('"');
-			i += 6;
-		}
-		else
-		{
-			ret.push_back(str[i]);
-			i++;
-		}
-	}
-	return ret;
-}
-
-
 // Print method for the messages in a Partition object.
 std::ostream& operator<<(std::ostream& out, const Partition& partition)
 {
@@ -127,42 +75,6 @@ std::ostream& operator<<(std::ostream& out, const Partition& partition)
 }
 
 
-// Print method for OperationDetailStatus.
-std::ostream& operator<<(std::ostream& out, const OperationDetailStatus od_status)
-{
-	switch (od_status)
-	{
-		case STATUS_NONE:     out << "NONE";     break;
-		case STATUS_EXECUTE:  out << "EXECUTE";  break;
-		case STATUS_SUCCESS:  out << "SUCCESS";  break;
-		case STATUS_ERROR:    out << "ERROR";    break;
-		case STATUS_INFO:     out << "INFO";     break;
-		case STATUS_WARNING:  out << "WARNING";  break;
-		default:                                 break;
-	}
-	return out;
-}
-
-
-// Print method for an OperationDetail object.
-std::ostream& operator<<(std::ostream& out, const OperationDetail& od)
-{
-	out << strip_markup(od.get_description());
-	Glib::ustring elapsed = od.get_elapsed_time();
-	if (! elapsed.empty())
-		out << "    " << elapsed;
-	if (od.get_status() != STATUS_NONE)
-		out << "  (" << od.get_status() << ")";
-	out << "\n";
-
-	for (unsigned int i = 0; i < od.get_childs().size(); i++)
-	{
-		out << *od.get_childs()[i];
-	}
-	return out;
-}
-
-
 // Printable file system type which meets the requirements for a Google Test name.
 // Use GParted's file system names except when they contains any non-alphabetic chars.
 // Reference:
@@ -183,8 +95,8 @@ const std::string test_fsname(FSType fstype)
 }
 
 
-// Function callable by INSTANTIATE_TEST_CASE_P() to get the file system type for printing
-// as part of the test name.
+// Function callable by INSTANTIATE_TEST_SUITE_P() to get the file system type for
+// printing as part of the test name.
 const std::string param_fsname(const ::testing::TestParamInfo<FSType>& info)
 {
 	return test_fsname(info.param);
@@ -239,7 +151,7 @@ const std::string param_fsname(const ::testing::TestParamInfo<FSType>& info)
 	}
 
 
-const Byte_Value IMAGESIZE_Default = 256*MEBIBYTE;
+const Byte_Value IMAGESIZE_Default = 320*MEBIBYTE;
 const Byte_Value IMAGESIZE_Larger  = 512*MEBIBYTE;
 
 
@@ -282,12 +194,12 @@ protected:
 };
 
 
-SupportedFileSystems* SupportedFileSystemsTest::s_supported_filesystems = NULL;
+SupportedFileSystems* SupportedFileSystemsTest::s_supported_filesystems = nullptr;
 const char*           SupportedFileSystemsTest::s_image_name            = "test_SupportedFileSystems.img";
 
 
 SupportedFileSystemsTest::SupportedFileSystemsTest()
- : m_fstype(GetParam()), m_fs_object(NULL), m_require_loopdev(false),
+ : m_fstype(GetParam()), m_fs_object(nullptr), m_require_loopdev(false),
    // Initialise top-level operation detail object with description ...
    m_operation_detail("Operation details:", STATUS_NONE)
 {
@@ -296,12 +208,12 @@ SupportedFileSystemsTest::SupportedFileSystemsTest()
 
 void SupportedFileSystemsTest::SetUp()
 {
-	ASSERT_TRUE(s_supported_filesystems != NULL) << __func__ << "(): TEST_BUG: File system interfaces not loaded";
+	ASSERT_TRUE(s_supported_filesystems != nullptr) << __func__ << "(): TEST_BUG: File system interfaces not loaded";
 
 	// Lookup file system interface object.
 	m_fs_object = s_supported_filesystems->get_fs_object(m_fstype);
-	ASSERT_TRUE(m_fs_object != NULL) << __func__ << "(): TEST_BUG: Interface object not found for file system "
-	                                 << test_fsname(m_fstype);
+	ASSERT_TRUE(m_fs_object != nullptr) << __func__ << "(): TEST_BUG: Interface object not found for file system "
+	                                    << test_fsname(m_fstype);
 }
 
 
@@ -331,7 +243,7 @@ void SupportedFileSystemsTest::TearDown()
 
 	unlink(s_image_name);
 
-	m_fs_object = NULL;
+	m_fs_object = nullptr;
 }
 
 
@@ -367,7 +279,7 @@ std::vector<FSType> SupportedFileSystemsTest::get_supported_fstypes()
 // Create the supported file system interface object.
 void SupportedFileSystemsTest::setup_supported_filesystems()
 {
-	if (s_supported_filesystems == NULL)
+	if (s_supported_filesystems == nullptr)
 	{
 		s_supported_filesystems = new SupportedFileSystems();
 
@@ -382,7 +294,7 @@ void SupportedFileSystemsTest::setup_supported_filesystems()
 void SupportedFileSystemsTest::teardown_supported_filesystems()
 {
 	delete s_supported_filesystems;
-	s_supported_filesystems = NULL;
+	s_supported_filesystems = nullptr;
 }
 
 
@@ -402,12 +314,7 @@ const std::string SupportedFileSystemsTest::create_loopdev(const std::string& im
 		return "";
 	}
 
-	// Strip trailing New Line.
-	size_t len = output.length();
-	if (len > 0 && output[len-1] == '\n')
-		output.resize(len-1);
-
-	return output;
+	return Utils::trim_trailing_new_line(output);
 }
 
 
@@ -454,7 +361,7 @@ void SupportedFileSystemsTest::reload_partition()
 
 	// Use libparted to get the sector size etc. of the image file.
 	PedDevice* lp_device = ped_device_get(m_dev_name.c_str());
-	ASSERT_TRUE(lp_device != NULL);
+	ASSERT_TRUE(lp_device != nullptr);
 
 	// Prepare partition object spanning whole of the image file.
 	m_partition.set_unpartitioned(m_dev_name,
@@ -465,7 +372,7 @@ void SupportedFileSystemsTest::reload_partition()
 	                              false);
 
 	ped_device_destroy(lp_device);
-	lp_device = NULL;
+	lp_device = nullptr;
 }
 
 
@@ -506,7 +413,6 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadUsage)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_FS_DOESNT_SUPPORT(read);
-	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_BTRFS);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
@@ -536,7 +442,6 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadLabel)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_FS_DOESNT_SUPPORT(read_label);
-	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_BTRFS);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
@@ -559,7 +464,6 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadUUID)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_FS_DOESNT_SUPPORT(read_uuid);
-	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_BTRFS);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
@@ -596,7 +500,11 @@ TEST_P(SupportedFileSystemsTest, CreateAndWriteLabel)
 	m_partition.set_filesystem_label("FIRST");
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
-	// Test writing a label is successful.
+	// Test clearing the label is successful.
+	m_partition.set_filesystem_label("");
+	ASSERT_TRUE(m_fs_object->write_label(m_partition, m_operation_detail)) << m_operation_detail;
+
+	// Test writing the label is successful.
 	m_partition.set_filesystem_label("SECOND");
 	ASSERT_TRUE(m_fs_object->write_label(m_partition, m_operation_detail)) << m_operation_detail;
 }
@@ -617,6 +525,32 @@ TEST_P(SupportedFileSystemsTest, CreateAndWriteUUID)
 }
 
 
+TEST_P(SupportedFileSystemsTest, CreateAndWriteUUIDAndReadLabel)
+{
+	SKIP_IF_FS_DOESNT_SUPPORT(create);
+	SKIP_IF_FS_DOESNT_SUPPORT(write_uuid);
+	SKIP_IF_FS_DOESNT_SUPPORT(read_label);
+	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
+	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
+
+	const char* fs_label = "TEST_LABEL";
+	create_image_file();
+	m_partition.set_filesystem_label(fs_label);
+	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
+
+	// Test writing a new random UUID is successful.
+	ASSERT_TRUE(m_fs_object->write_uuid(m_partition, m_operation_detail)) << m_operation_detail;
+
+	// Test reading the label is successful and it hasn't changed.
+	reload_partition();
+	m_fs_object->read_label(m_partition);
+	EXPECT_STREQ(fs_label, m_partition.get_filesystem_label().c_str());
+
+	// Test messages from read operation are empty or print them.
+	EXPECT_TRUE(m_partition.get_messages().empty()) << m_partition;
+}
+
+
 TEST_P(SupportedFileSystemsTest, CreateAndCheck)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
@@ -687,48 +621,16 @@ TEST_P(SupportedFileSystemsTest, CreateAndShrink)
 // Instantiate the test case so every test is run for the specified file system types.
 // Reference:
 // *   Google Test, Advanced googletest Topics, How to Write Value-Parameterized Tests
-//     https://github.com/google/googletest/blob/v1.8.x/googletest/docs/advanced.md#how-to-write-value-parameterized-tests
-INSTANTIATE_TEST_CASE_P(My,
-                        SupportedFileSystemsTest,
-                        ::testing::ValuesIn(SupportedFileSystemsTest::get_supported_fstypes()),
-                        param_fsname);
+//     https://github.com/google/googletest/blob/v1.10.x/googletest/docs/advanced.md#how-to-write-value-parameterized-tests
+INSTANTIATE_TEST_SUITE_P(My,
+                         SupportedFileSystemsTest,
+                         ::testing::ValuesIn(SupportedFileSystemsTest::get_supported_fstypes()),
+                         param_fsname);
 
 
 }  // namespace GParted
 
 
-// Re-execute current executable using xvfb-run so that it provides a virtual X11 display.
-void exec_using_xvfb_run(int argc, char** argv)
-{
-	// argc+2 = Space for "xvfb-run" command, existing argc strings plus NULL pointer.
-	size_t size = sizeof(char*) * (argc+2);
-	char** new_argv = (char**)malloc(size);
-	if (new_argv == NULL)
-	{
-		fprintf(stderr, "Failed to allocate %lu bytes of memory.  errno=%d,%s\n",
-			(unsigned long)size, errno, strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	new_argv[0] = strdup("xvfb-run");
-	if (new_argv[0] == NULL)
-	{
-		fprintf(stderr, "Failed to allocate %lu bytes of memory.  errno=%d,%s\n",
-		        (unsigned long)strlen(new_argv[0])+1, errno, strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	// Copy argv pointers including final NULL pointer.
-	for (unsigned int i = 0; i <= (unsigned)argc; i++)
-		new_argv[i+1] = argv[i];
-
-	execvp(new_argv[0], new_argv);
-	fprintf(stderr, "Failed to execute '%s %s ...'.  errno=%d,%s\n", new_argv[0], new_argv[1],
-		errno, strerror(errno));
-	exit(EXIT_FAILURE);
-}
-
-
 // Custom Google Test main().
 // Reference:
 // *   Google Test, Primer, Writing the main() function
@@ -736,18 +638,11 @@ void exec_using_xvfb_run(int argc, char** argv)
 int main(int argc, char** argv)
 {
 	printf("Running main() from %s\n", __FILE__);
-
-	const char* display = getenv("DISPLAY");
-	if (display == NULL)
-	{
-		printf("DISPLAY environment variable unset.  Executing 'xvfb-run %s ...'\n", argv[0]);
-		exec_using_xvfb_run(argc, argv);
-	}
-	printf("DISPLAY=\"%s\"\n", display);
+	GParted::ensure_x11_display(argc, argv);
 
 	// Initialise threading in GParted to allow FileSystem interface classes to
-	// successfully use Utils:: and Filesystem::execute_command().  Must be before
-	// InitGoogleTest().
+	// successfully use Utils:: and OperationDetail::execute_command().  Must be
+	// before InitGoogleTest().
 	GParted::GParted_Core::mainthread = Glib::Thread::self();
 	Gtk::Main gtk_main = Gtk::Main();
 
